from __future__ import annotations

from dataclasses import dataclass


@dataclass
class Color:
    red: int
    green: int
    blue: int
    opacity: float = 1.0

    @staticmethod
    def from_hex(hexString: str) -> Color:
        hexString = hexString.lstrip('#')
        if len(hexString) == 3:
            parts = [int(f'{hexString[i:i + 1]}{hexString[i:i + 1]}', 16) for i in range(0, 3)]
        elif len(hexString) == 6:
            parts = [int(hexString[i:i + 2], 16) for i in (0, 2, 4)]
        elif len(hexString) == 8:
            parts = [int(hexString[i:i + 2], 16) for i in (0, 2, 4, 6)]
        else:
            raise ValueError('Provided hex string has invalid format!')

        return Color(parts[0], parts[1], parts[2], (parts[3] / 255.0) if len(parts) == 4 else 1.0)

    def to_hex(self) -> str:
        opacity = int(self.opacity * 255)
        return '#{:02X}{:02X}{:02X}{:02X}'.format(self.red, self.green, self.blue, opacity)

    def to_hex_without_opacity(self) -> str:
        return '#{:02X}{:02X}{:02X}'.format(self.red, self.green, self.blue)

    def to_rgba(self) -> str:
        return f'rgba({self.red}, {self.green}, {self.blue}, {self.opacity})'

    def to_rgb(self) -> str:
        return f'rgb({self.red}, {self.green}, {self.blue})'
